<?php

/**
 * Gets details of the latest flight arrival/departures at Male' International 
 * Airport.
 *
 * Usage example:
 * $flights = new flightinfo();
 * $flights->getInfo( 'arrivals' );
 * $flights->sortBy( 'estimated' );
 * echo $flights->updatetime;
 * print_r( $flights->flights );
 *
 *
 * PHP version 4 and 5
 *
 * LICENSE: Freeware. Distribution and modification allowed with credits intact.
 *
 * @author 		Jawish Hameed <jaa@technova.com.mv>
 * @version 	0.1 (2006-12-13) 
 * @link 		http://jaa.technova.com.mv/
 */


// {{{ FlightInfo class

/**
 * Processes flight information data file and makes it available as easily
 * accessible properties of an object
 *
 * @package 	flightinfo
 * @author 		Jawish Hameed <jaa@technova.com.mv>
 * @version 	0.1 (2006-12-13)
 */
class flightinfo
{
	// {{{ properties
	
	/**
	 * Time of last update to flight data
	 * Time is in unix timestamp format
	 *
	 * @var integer
	 * @access public
	 */
	var $updatedtime;
	
	/**
	 * Date for the listing
	 *
	 * @var integer
	 * @access public
	 */
	var $day;
	
	/**
	 * Flights and their details as a list of 'flight' objects
	 *
	 * @var array
	 * @access public
	 */
	var $flights;

	// }}}
	
	
	// {{{ class constructor
	
	/**
	 * Initializes object and optionally fetches flight data
	 *
	 * @param string	$type		set to fetch 'arrivals' or 'departures' data
	 *
	 * @access public
	 */
	function flightinfo( $type = '' )
	{
		// get info if type parameter specified
		if ( $type != '' ) $this->getInfo( $type );
	}
	
	// }}
	
	
	// {{{ getInfo
	
	/**
	 * Gets flight data as specified
	 *
	 * @param string	$type		set to fetch 'arrivals' or 'departures' data
	 *
	 * @access public
	 */
	function getInfo( $type = 'arrivals' )
	{
		// check whether to get arrival or departure data
		if ( $type == 'arrivals' ):
			$link = 'http://www.fis.com.mv/cgi-bin/webfids.pl?webfids_type=arrivals';
		else:
			$link = 'http://www.fis.com.mv/cgi-bin/webfids.pl?webfids_type=departures';
		endif;
		
		// setup patterns
		$patUpdated = '|<td class="updatedtime">UPDATED: (.*)</td>|';
		$patDay = '|<td colspan="6" class="summaryhead">.*&nbsp;(.*)</td>|';
		$patEntries = '|<td colspan="6" class="summaryhead">(.*)<td colspan="6" class="summaryhead">|Us';
		$patEntry = '|<tr class="schedulerow.*" valign="top">(.*)</tr>|Us';
		
		// fetch data from FIS site
		$data = file_get_contents( $link );
		
		// extract update time
		preg_match( $patUpdated, $data, $matches );
		$matches[ 1 ] = str_replace( array( '|', ',' ), '', $matches[ 1 ] );
		$this->updatedtime = strtotime( $matches[ 1 ] );
		
		// extract day for listing
		preg_match( $patDay, $data, $matches );
		$this->day = strtotime( $matches[ 1 ] );
		
		// locate entries for first day
		$entries = array();
		if ( preg_match( $patEntries, $data, $matches ) ):
			
			unset( $data );
			
			// extract entries
			preg_match_all( $patEntry, $matches[ 1 ], $matches );
			
			// process the entries
			foreach ( $matches[ 1 ] as $match ):
				
				$fdata = array();
				
				// extract the fields
				preg_match_all( '|<td class="(.*)".*>(.*)</td>|Us', $match, $temp );
				
				// get airline name and id
				preg_match( '|alt="(.*)"|', $temp[ 2 ][ 0 ], $fdata[ 'airlinename' ] );
				preg_match( '|images/(.*)\.|', $temp[ 2 ][ 0 ], $fdata[ 'airlineid' ] );
				$fdata[ 'airlinename' ] = $fdata[ 'airlinename' ][ 1 ];
				$fdata[ 'airlineid' ] = strtoupper( $fdata[ 'airlineid' ][ 1 ] );
				
				// get flightid
				$fdata[ 'flightid' ] = $temp[ 2 ][ 1 ];
				
				// get route
				$delim = ( $type == 'arrivals' ) ? ' / ' : ' \ ';
				$fdata[ 'route' ] = explode( $delim, $temp[ 2 ][ 2 ] );
				
				// get time
				$fdata[ 'scheduled' ] = $temp[ 2 ][ 3 ];
				
				// get estimated time
				$fdata[ 'estimated' ] = ( $temp[ 2 ][ 4 ] == '&nbsp;' ) ? '' : $temp[ 2 ][ 4 ];
				
				// get status
				$fdata[ 'status' ] = ( $temp[ 2 ][ 5 ] == '&nbsp;' ) ? '' : $temp[ 2 ][ 5 ];
				$fdata[ 'status' ] = strip_tags( $fdata[ 'status' ] );
				
				// add data to flight listing
				$this->flights[] = $fdata;
			
			endforeach;
			
		endif;
	}
	
	// }}}
	
	
	// {{{ sortBy
	
	/**
	 * Sort the flights by the given field in given order
	 *
	 * @param string	$key	field name to sort by
	 * @param integer	$order	order: 1 = ascending, -1 = descending
	 *
	 * @access public
	 */
	function sortBy( $key, $order = 1 )
	{
		// do nothing if no flight data available
		if ( !is_array( $this->flights ) ) return;
		
		// get the column data for set field
		$sortcol = array();
		foreach ( $this->flights as $flight ):
			$sortcol[] = $flight[ $key ];
		endforeach;
		
		// check for sort order type
		$order = ( $order == 1 ) ? SORT_ASC : SORT_DESC;
		
		// do sort
		array_multisort(	$sortcol, $order, SORT_REGULAR,
							$this->flights, $order, SORT_REGULAR
							);
	}
	
	// }}}
	
}

// }}}

?>